/*
 * board/cmx300/boardvoltage.S
 *
 * Configuration settings for the CompuLab CM-X300 device
 *
 * (C) Copyright 2007 CompuLab, Igor Vaisbein
 *
 * (C) Copyright 2009, 2010
 * Igor Grinberg, CompuLab, grinberg@compulab.co.il
 *
 * This was originally from the adsvix board
 * (C) Copyright 2006 Koan sas - Bergamo - ITALIA
 *                    Marco Cavallini, (MCK) <m.cavallini @ koansoftware.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <asm/arch/pxa-regs.h>
#include <da9030.h>

#define LDO2_3_VAL	0xF0
#define LDO4_5_VAL	0xF0
/*
 * SIMCP control bits are set to <1,0> (0x2) to guarantee VLDO6
 * voltage is always higher then 2.9V
 */
#define LDO6_SIMCP_VAL	0x2F
#define LDO7_8_VAL	0xAF
#define LDO9_12_VAL	0xFA
#define LDO_10_11_VAL	0xFF
#define LDO_14_16_VAL	0x30
#define LDO_18_19_VAL	0x0F

.globl init_board_voltage
.globl initializei2c

reset_counter:
	.word 0x00000003

/*
 * Initialize the PWR_I2C interface
 */
initializei2c:
// 		ldr	r2, =CKEN
// 		ldr	r3, [r2]
// 		orr	r3, r3, #CKEN15_PWRI2C
// 		str	r3, [r2]
//
// 		ldr	r2, =PCFR
// 		ldr	r3, [r2]
// 		orr	r3, r3, #PCFR_PI2C_EN
// 		str	r3, [r2]
//
// 		/* delay for about 250msec
// 		 */
// 		ldr	r3, =OSCR
// 		mov	r2, #0
// 		str	r2, [r3]
// 		ldr	r1, =0xC0000
//
// 1:
// 		ldr	r2, [r3]
// 		cmp	r1, r2
// 		bgt	1b

	ldr	r0, =PWRICR
	ldr	r1, [r0]
	bic	r1, r1, #(ICR_MA | ICR_START | ICR_STOP)
	str	r1, [r0]

	orr	r1, r1, #ICR_UR
	str	r1, [r0]

	ldr	r2, =PWRISR
	ldr	r3, =0x7ff
	str	r3, [r2]

	bic	r1, r1, #ICR_UR
	str	r1, [r0]

	mov	r1, #(ICR_GCD | ICR_SCLE)
	str	r1, [r0]

	orr	r1, r1, #ICR_IUE
	str	r1, [r0]

	orr	r1, r1, #ICR_FM
	str	r1, [r0]

	/* delay for about 0.25msec */
	ldr	r3, =OSCR
	mov	r2, #0
	str	r2, [r3]
	ldr	r1, =0xC00

1:
	ldr	r2, [r3]
	cmp	r1, r2
	bgt	1b
	mov	pc, lr

led_blink:
	/* r0 - the first argument stores the number of times to blink */
	mov	r3, r0
1:
	/* turn the green led on for cm-x300 rev 1.2 and rev 1.3 */
	ldr	r2, =0x9000		/* gpio 79 and 76 */

	ldr	r0, =GPDR2		/* r1 - gpdr2 */
	ldr	r1, [r0]
	orr	r1, r1, r2		/* set gpio 79 and 76 as output */
	str	r1, [r0]		/* update gpdr2 */

	ldr	r0, =GPCR2		/* r1 - gpcr2 */
	ldr	r1, [r0]
	orr	r1, r1, r2		/* clear gpio 79 and 76 to light up the led */
	str	r1, [r0]		/* update gpcr2 */

	/* delay for about 10 msec */
	ldr	r7, =OSCR
	mov	r8, #0
	str	r8, [r7]
	ldr	r1, =0x33000
2:
	ldr	r8, [r7]
	cmp	r1, r8
	bgt	2b

	ldr	r0, =GPSR2		/* r1 - gpsr2 */
	ldr	r1, [r0]
	orr	r1, r1, r2		/* set gpio 79 and 76 to turn off the led */
	str	r1, [r0]		/* update gpsr2 */
	
	/* delay for about 10 msec */
	ldr	r7, =OSCR
	mov	r8, #0
	str	r8, [r7]
	ldr	r1, =0x33000
2:
	ldr	r8, [r7]
	cmp	r1, r8
	bgt	2b

	/* blink the given number of times */
	subs	r3, r3, #1
	bne	1b

	mov	pc, lr

init_board_voltage:

	mov	r4, lr
	bl	initializei2c

retry_board_voltage:

 	/* now set the correct core voltage */
	mov	r0, #BUCK2_DVC1
	mov	r1, #(BUCK2_GO | BUCK2_TRIM_1V375)
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* set the correct SRAM voltage */
	mov	r0, #LDO_14_16
	mov	r1, #LDO_14_16_VAL
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* delay for about msec for the voltage to stabilize */
	ldr	r3, =OSCR
	mov	r2, #0
	str	r2, [r3]
	ldr	r1, =0xC0
1:
	ldr	r2, [r3]
	cmp	r1, r2
	bgt	1b

	/* Disable all unneeded LDOs */
	/* additional preparations to disable LDO10 */
	mov	r0, #STARTUP_CONTROL
	mov	r1, #STARTUP_CTL_LDO11_START
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* Disable LDO10 (AC97) and NC: 13, 14, 15, 17 */
	mov	r0, #REG_CONTROL_1_17
	mov	r1, #(RC1_LDO16_EN | RC1_LDO11_EN | RC1_BUCK2_EN)
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* Disable SIMCP */
	mov	r0, #REG_CONTROL_2_18
	mov	r1, #(RC2_LDO19_EN | RC2_LDO18_EN)
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage	/* goto Exit */

	/*
	 * Disable LDO: 1 (NC), 2 (NC), 3 (LAN), 4 (USB ULPI),
	 * 		5 (BT), 7 (USB)
	 */
	mov	r0, #REG_CONTROL_1_97
	mov	r1, #(RC3_LDO6_EN | RC3_BUCK_EN )
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/*
	 * Disable LDO: 8 (NC), 9 (NC), 10 (AC97), 12 (WIFI),
	 * 		14 (NC), SIMCP, Sleep
	 */
	mov	r0, #REG_CONTROL_2_98
	mov	r1, #RC4_LDO11_EN
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* Set the RS-232 Voltage */
	mov	r0, #LDO6_SIMCP
	mov	r1, #LDO6_SIMCP_VAL
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* Set the Data Flash and AC'97 Voltages */
	mov	r0, #LDO_10_11
	mov	r1, #LDO_10_11_VAL
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage


	/* Set the BLUETOOTH and COMMS Voltages */
	mov	r0, #LDO4_5
	mov	r1, #LDO4_5_VAL
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* Set the WLAN Voltage */
	mov	r0, #LDO9_12
	mov	r1, #LDO9_12_VAL
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* Set the I2C GPIO Voltage */
	mov	r0, #LDO_18_19
	mov	r1, #LDO_18_19_VAL
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* Set the USB2 Voltage */
	mov	r0, #LDO7_8
	mov	r1, #LDO7_8_VAL
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* Set the LAN Voltage */
	mov	r0, #LDO2_3
	mov	r1, #LDO2_3_VAL
	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage

	/* Enable LAN */
	mov	r0, #REG_CONTROL_1_97
	mov	r1, #(RC3_LDO6_EN | RC3_LDO3_EN | RC3_BUCK_EN)
  	bl	da9030_set_reg
	cmp	r8, #1			/* Test for Error */
	beq	error_board_voltage
	
	/* delay for about 1msec for the volatage to stabilize */
	ldr	r3, =OSCR
	mov	r2, #0
	str	r2, [r3]
	ldr	r1, =0x3300
1:
	ldr	r2, [r3]
	cmp	r1, r2
	bgt	1b

	mov	pc, r4

error_board_voltage:

	/* If the reset counter reached zero - try to reset cpu */
	ldr	r0, =reset_counter
	ldr	r1, [r0]
	tst	r1, #1	/* check if alredy zero */
	bleq	reset_cpu_board_voltage
	sub	r1, r1, #1
   
	str	r1, [r0]
	bl	initializei2c
	b	retry_board_voltage

reset_cpu_board_voltage:
	/* blink 5 times */
	mov	r0, #5
	bl	led_blink
	b	reset_cpu
