/*
 * board/cmx300/cmx300_common.h
 *
 * Common settings and macros for the CompuLab CM-X300 module
 *
 * (C) Copyright 2009
 * Igor Grinberg, CompuLab, grinberg@compulab.co.il
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CM_X300_COMMON__
#define __CM_X300_COMMON__

#include <asm/arch/hardware.h>

#define CPUID_ID	0
#define CPUID_CACHETYPE	1
#define CPUID_TCM	2
#define CPUID_TLBTYPE	3

#define SERIAL_ENV_VAR_NAME			"serial#"
#define ADVANCED_POWER_MODE_ENV_VAR_NAME	"adv_pwr_mode"

#define mdelay(n)	({unsigned long msec = (n); while (msec--) udelay(1000); })

#define stringify(x)	#x

#define read_cpuid(reg)							\
	({								\
		unsigned int __val;					\
		asm("mrc	p15, 0, %0, c0, c0, " stringify((reg))	\
	: "=r" (__val)					\
	:						\
	: "cc");					\
		__val;							\
})

#define cpu_is_pxa300()							\
	({								\
		unsigned int _id = (read_cpuid(CPUID_ID)) >> 4 & 0xfff;	\
		_id == 0x688;						\
})

#define cpu_is_pxa310()							\
	({								\
		unsigned int _id = (read_cpuid(CPUID_ID)) >> 4 & 0xfff;	\
		_id == 0x689;						\
})

/* Optimization barrier */
/* The "volatile" is due to gcc bugs */
#define cmx300_cpu_relax() __asm__ __volatile__("": : :"memory")


int is_adv_pwr_mod_enabled(void);

#endif /* __CM_X300_COMMON__ */
