/*
 * board/cmx300/da9030_fixup.c
 *
 * C-Wrapper for the da9030 register get/set assembly interface
 * function for da9030.
 *
 * (C) Copyright 2010
 * Yulia Vilensky CompuLab, vilensky@compulab.co.il
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "da9030.h"

extern void da9030_set_reg(u32 reg, u32 val);
extern u32 da9030_get_reg(u32 reg);

static int da9030_set(u32 reg, u32 val);
static int da9030_get(u32 reg, u32 *val);

static struct da9030_ops ops = {
	.set_reg = da9030_set,
	.get_reg = da9030_get,
};


struct da9030_ops *da9030_get_ops(void)
{
	return &ops;
}

/*
 * This is a wrapper function for the assembly da9030_set_reg.
 * It stores all the callee-saved registers, and then restores
 * them - the thing that the assembly function doesn't take
 * care of.
 */
static int da9030_set(u32 reg, u32 val)
{
	int res;

	__asm__ __volatile__(
	/* store the callee registers on stack to be restored later */
	"stmfd    sp!,{r4-r12}\n\t"
	/* update the arguments - just to be sure */
	"mov	r0, %[address]\n\t"
	"mov	r1, %[value]\n\t"
	"bl	da9030_set_reg\n\t"
	/* the return code is in r8 */
	"mov	%[result], r8\n\t"
	/* restore the registers */
	"ldmfd  sp!,{r4-r12}\n\t"
	/* return code will be stored in res variable */
	: [result] "=r" (res)
	/* inputs are reg and val mapped to address and value */
	: [address] "r" (reg), [value] "r" (val)
	/* things that can be changed, list for compiler */
	: "cc", "r0", "r1", "r2", "r3", "memory");

	return res;
}

/*
 * This is a wrapper function for the assembly da9030_get_reg.
 * It stores all the callee-saved registers, and then restores
 * them - the thing that the assembly function doesn't take
 * care of.
 */
static int da9030_get(u32 reg, u32 *val)
{
	int res;
	u32 value;

	__asm__ __volatile__(
	/* store the callee registers on stack to be restored later */
	"stmfd    sp!,{r4-r12}\n\t"
	/* update the input argument - just to be sure */
	"mov	r0, %[address]\n\t"
	"bl	da9030_get_reg\n\t"
	/* the register value is in r0 */
	"mov	%[value], r0\n\t"
	/* the return code is in r8 */
	"mov	%[result], r8\n\t"
	/* restore the registers */
	"ldmfd  sp!,{r4-r12}\n\t"
	/* outputs list mapping*/
	: [result] "=r" (res), [value] "=r" (value)
	/* input mapping*/
	: [address] "r" (reg)
	/* things that can be changed, list for compiler */
	: "cc", "r1", "r2", "r3", "memory");

	*val = value & 0xff;
	return res;
}
