/*
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <asm/arch/pxa-regs.h>


GPIOREGS_PHYSICAL_BASE     = 	0x40E00000
MFPR_PHYSICAL_BASE         =    0x40E10000

GPLR0_offset       =     0x000   /* GPIO Level registers*/
GPLR1_offset       =     0x004
GPLR2_offset       =     0x008
GPLR3_offset       =     0x100

GPDR0_offset       =     0x00C   /* GPIO Direction registers*/
GPDR1_offset       =     0x010
GPDR2_offset       =     0x014
GPDR3_offset       =     0x10C

GPSR0_offset       =     0x018   /* GPIO Set registers */
GPSR1_offset       =     0x01C
GPSR2_offset       =     0x020
GPSR3_offset       =     0x118

GPCR0_offset       =     0x024   /* GPIO Clear registers */
GPCR1_offset       =     0x028
GPCR2_offset       =     0x02C
GPCR3_offset       =     0x124

GPSR0_value        =     0x00000000
GPSR1_value        =     0x00000000
GPSR2_value        =     0x00000000
GPSR3_value        =     0x00000000
GPCR0_value        =     0xFFFFFFFF
GPCR1_value        =     0xFFFFFFFF
GPCR2_value        =     0xFFFFFFFF
GPCR3_value        =     0xFFFFFFFF
GPDR0_value        =     0x00000000
GPDR1_value        =     0x00000000
GPDR2_value        =     0x00000000
GPDR3_value        =     0x00000000

.globl is_pxa300
is_pxa300:
	.word 0x00000000

MFPR_init:
/* Set all GPIO Multi Function Pin Registers (MFPRxx) to the values set */
/* by the include file referenced in the code below. Since offsets to */
/* MFPRs are not in a linear sequence, an offset lookup table below) is used */
	ldr	r0, =MFPR_PHYSICAL_BASE		/* MFPR base address */
	add	r4, pc, #MFPR_data - (.+8)	/* Address of data for MFPR settings */

	mrc	p15, 0, r2, c0, c0, 0		/* read the CPUID register */
	ldr	r3, =0xffff000f			/* CPUID mask */
	bic	r2, r2, r3
	ldr	r3, =0x6880			/* PXA300 id */
	cmp	r2, r3				/* eq=PXA300, ne=PXA310 */
	moveq	r1, #1				/* PXA300 */
	movne	r1, #0				/* PXA310 */

	/* store the processor type in a global variable */
	ldr	r2, =is_pxa300
	str	r1, [r2]

	teq	r1, #1
	ldreq	r2, =PXA300_MFPR_offset_table	/* Address of PXA300 MFPR offset table */
	ldrne	r2, =PXA310_MFPR_offset_table	/* Address of PXA310 MFPR offset table */
	ldr	r3, =MFPR_data			/* Address of PXA3XX MFPR data */

	sub	r2, r3, r2			/* Calculate distance between the two */
	sub	r2, r4, r2			/* R2 now contains address of the table */
xlli_m0:
	ldr	r3, [r2], #4			/* Get next offset value */
	cmp	r3, #0				/* At end of table? */
	beq	xlli_m1				/* Branch if end of table <Offset = Zero> */
	ldr	r1, [r4], #4			/* Fetch MFPR data from user table (below) */
	str	r1, [r0, r3]			/* Write MFPR data to MFPR */
	b	xlli_m0				/* Get next offset */
/* Perform a readback & return to caller */
xlli_m1:
@	sub	r2, r2, #2		/* Point back to MFPR for last table entry */
@	ldr	r3, [r2]		/* Get the last offset value */
@	ldr	r4, [r0, r3]		/* Read back to insure writes are complete */
	mov	pc, lr

/* This lookup table provides offsets to the Multi Function */
/* Pin Registers (MFPRs), in ascending order, from GPIO_0 to GPIO_127 */
/* followed by the GPIO_0_2 to GPIO_6_2 MFPRs for Monahans_L (PXA300) */
PXA300_MFPR_offset_table:
	.word	0x0B4, 0x0B8, 0x0BC, 0x27C, 0x280, 0x284, 0x288, 0x28C	/* MFPRs 0->7 */
	.word	0x290, 0x294, 0x298, 0x29C, 0x2A0, 0x2A4, 0x2A8, 0x2AC	/* MFPRs 8->15 */
	.word	0x2B0, 0x2B4, 0x2B8, 0x2BC, 0x2C0, 0x2C4, 0x2C8, 0x2CC	/* MFPRs 16->23 */
	.word	0x2D0, 0x2D4, 0x2D8, 0x400, 0x404, 0x408, 0x40C, 0x410	/* MFPRs 24->31 */
	.word	0x414, 0x418, 0x41C, 0x420, 0x424, 0x428, 0x42C, 0x430	/* MFPRs 32->39 */
	.word	0x434, 0x438, 0x43C, 0x440, 0x444, 0x448, 0x44C, 0x450	/* MFPRs 40->47 */
	.word	0x454, 0x458, 0x45C, 0x460, 0x464, 0x468, 0x46C, 0x470	/* MFPRs 48->55 */
	.word	0x474, 0x478, 0x47C, 0x480, 0x484, 0x488, 0x48C, 0x490	/* MFPRs 56->63 */
	.word	0x494, 0x498, 0x49C, 0x4A0, 0x4A4, 0x4A8, 0x4AC, 0x4B0	/* MFPRs 64->71 */
	.word	0x4B4, 0x4B8, 0x4BC, 0x4C0, 0x4C4, 0x4C8, 0x4CC, 0x4D0	/* MFPRs 72->79 */
	.word	0x4D4, 0x4D8, 0X4DC, 0X4E0, 0x4E4, 0x4E8, 0x4EC, 0x4F0	/* MFPRs 80->87 */
	.word	0x4F4, 0x4F8, 0x4FC, 0x500, 0x504, 0x508, 0x50C, 0x510	/* MFPRs 88->95 */
	.word	0x514, 0x518, 0x51C, 0x600, 0x604, 0x608, 0x60C, 0x610	/* MFPRs 96->103 */
	.word	0x614, 0x618, 0x61C, 0x620, 0x624, 0x628, 0x62C, 0x630	/* MFPRs 104->111 */
	.word	0x634, 0x638, 0x63C, 0x640, 0x644, 0x648, 0x64C, 0x650	/* MFPRs 112->119 */
	.word	0x654, 0x658, 0x65C, 0x660, 0x664, 0x668, 0x66C, 0x670	/* MFPRs 120->127 */
	.word	0x674, 0x678, 0x2DC, 0x2E0, 0x2E4, 0x2E8, 0x2EC		/* MFPRs 0_2->6_2 */
	.word	0x000		/*  0x000 marks the end of the table */
	.align 4 /* Ensures 32-bit allignment */

/* A user table is inserted below (as an INCLUDE file) to set up the */
/* MFPRs at platform boot time. The MFPR values must be 32-bit values, in */
/* sequence, starting with the MFPR value for GPIO_0 and ending with */
/* GPIO_127 (128 entries) followed by 18 more entries for GPIO_0_2 to */
/* GPIO_6_2.  The table provided with the XLLI code DOES NOT SET UP THE */
/* ALTERNATE:BITS.  As provided, the MFPR table supplied with XLLI along */
/* with the GPIO register data provided in the platform include file, */
/* will set the GPIO direction, the pin output level (if the pin is */
/* configured as an output), pull up/down resistors and drive */
/* strength. The configuration of the GPIOs after the GPIO_init code has */
/* completed is designed to place the platform I/Os in a minimum current */
/* drain mode.  After GPIO_init has been called, it is left up to the */
/* user to configure the alternate function bits as drivers are */
/* loaded/initialized. If referencing GPIOs by GPIO number, the user may */
/* want to take advantage of the offset table above since it is alreay */
/* loaded into memory. */
MFPR_data:
	.long	0x0000A040	/* GPIO_0		- Pull Down Enabled */
	.long	0x0000A041	/* GPIO_1		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_2		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_3		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_4		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_5		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_6		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_7		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_8		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_9		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_10		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_11		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_12		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_13		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_14		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_15		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_16		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_17		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_18		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_19		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_20		- Pull Down Enabled */
	.long	0x00000041	/* GPIO_21		- Enable I2C */
	.long	0x00000041	/* GPIO_22		- Enable I2C */
	.long	0x0000A040	/* GPIO_23		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_24		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_25		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_26		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_27		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_28		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_29		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_30		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_31		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_32		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_33		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_34		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_35		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_36		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_37		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_38		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_39		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_40		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_41		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_42		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_43		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_44		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_45		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_46		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_47		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_48		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_49		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_50		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_51		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_52		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_53		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_54		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_55		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_56		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_57		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_58		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_59		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_60		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_61		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_62		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_63		- Pull Up enabled */
	.long	0x0000A040	/* GPIO_64		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_65		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_66		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_67		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_68		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_69		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_70		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_71		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_72		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_73		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_74		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_75		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_76		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_77		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_78		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_79		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_80		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_81		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_82		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_83		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_84		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_85		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_86		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_87		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_88		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_89		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_90		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_91		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_92		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_93		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_94		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_95		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_96		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_97		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_98		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_99		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_100		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_101		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_102		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_103		- Pull Up Enabled */
	.long	0x000080C0	/* GPIO_104		- Pull Up Enabled (Output Low on sleep) */
	.long	0x0000C040	/* GPIO_105		- Pull Up Enabled */
	.long	0x0000C040	/* GPIO_106		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_107		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_108		- Pull Down Enabled */
	.long	0x0000C041	/* GPIO_109		- AF #1 UART3_TTL_TXD */
	.long	0x0000A041	/* GPIO_110		- AF #1 UART3_TTL_RXD */
	.long	0x0000A040	/* GPIO_111		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_112		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_113		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_114		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_115		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_116		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_117		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_118		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_119		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_120		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_121		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_122		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_123		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_124		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_125		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_126		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_127		- Pull Down Enabled */
	.long	0x0000C040	/* GPIO_0_2		- Pull Up Enabled */
	.long	0x0000A040	/* GPIO_1_2		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_2_2		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_3_2		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_4_2		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_5_2		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_6_2		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_7_2		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_8_2		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_9_2		- Pull Down Enabled */
	.long	0x0000A040	/* GPIO_10_2	- Pull Down Enabled */

/* This lookup table provides offsets to the Multi Function */
/* Pin Registers (MFPRs), in ascending order, from GPIO_0 to GPIO_127 */
/* followed by the GPIO_0_2 to GPIO_10_2 MFPRs for Monahans_LV (PXA310) */
PXA310_MFPR_offset_table:
	.word	0x0B4, 0x0B8, 0x0BC, 0x27C, 0x280, 0x284, 0x288, 0x28C	/* MFPRs 0->7 */
	.word	0x290, 0x294, 0x298, 0x29C, 0x2A0, 0x2A4, 0x2A8, 0x2AC	/* MFPRs 8->15 */
	.word	0x2B0, 0x2B4, 0x2B8, 0x2BC, 0x2C0, 0x2C4, 0x2C8, 0x2CC	/* MFPRs 16->23 */
	.word	0x2D0, 0x2D4, 0x2D8, 0x400, 0x404, 0x408, 0x418, 0x41C	/* MFPRs 24->31 */
	.word	0x420, 0x424, 0x428, 0x42C, 0x430, 0x434, 0x438, 0x43C	/* MFPRs 32->39 */
	.word	0x440, 0x444, 0x448, 0x44C, 0x450, 0x454, 0x458, 0x45C	/* MFPRs 40->47 */
	.word	0x460, 0x464, 0x468, 0x46C, 0x470, 0x474, 0x478, 0x47C	/* MFPRs 48->55 */
	.word	0x480, 0x484, 0x488, 0x48C, 0x490, 0x494, 0x498, 0x49C	/* MFPRs 56->63 */
	.word	0x4A0, 0x4A4, 0x4A8, 0x4AC, 0x4B0, 0x4B4, 0x4B8, 0x4BC	/* MFPRs 64->71 */
	.word	0x4C0, 0x4C4, 0x4C8, 0x4CC, 0x4D0, 0x4D4, 0x4D8, 0X4DC	/* MFPRs 72->79 */
	.word	0X4E0, 0x4E4, 0x4E8, 0x4EC, 0x4F0, 0x4F4, 0x4F8, 0x4FC	/* MFPRs 80->87 */
	.word	0x500, 0x504, 0x508, 0x50C, 0x510, 0x514, 0x518, 0x51C	/* MFPRs 88->95 */
	.word	0x520, 0x524, 0x528, 0x600, 0x604, 0x608, 0x60C, 0x610	/* MFPRs 96->103 */
	.word	0x614, 0x618, 0x61C, 0x620, 0x624, 0x628, 0x62C, 0x630	/* MFPRs 104->111 */
	.word	0x634, 0x638, 0x63C, 0x640, 0x644, 0x648, 0x64C, 0x650	/* MFPRs 112->119 */
	.word	0x654, 0x658, 0x65C, 0x660, 0x664, 0x668, 0x66C, 0x670	/* MFPRs 120->127 */
	.word	0x674, 0x678, 0x2DC, 0x2E0, 0x2E4, 0x2E8, 0x2EC, 0x52C	/* MFPRs 0_2->7_2 */
	.word	0x530, 0x534, 0x538					/* MFPRs 8_2->10_2 */
	.word	0x000		/*  0x000 marks the end of the table */
	.align 4 /* Ensures 32-bit allignment */

GPIO_init:
	ldr	r0, =GPIOREGS_PHYSICAL_BASE

	ldr	r1, =GPSR0_value
	str	r1, [r0, #GPSR0_offset]

	ldr	r2, =GPSR1_value
	str	r2, [r0, #GPSR1_offset]

	ldr	r1, =GPSR2_value
	str	r1, [r0, #GPSR2_offset]

	ldr	r2, =GPSR3_value
	str	r2, [r0, #GPSR3_offset]

	ldr	r1, =GPCR0_value
	str	r1, [r0, #GPCR0_offset]

	ldr	r2, =GPCR1_value
	str	r2, [r0, #GPCR1_offset]

	ldr	r1, =GPCR2_value
	str	r1, [r0, #GPCR2_offset]

	ldr	r2, =GPCR3_value
	str	r2, [r0, #GPCR3_offset]

	ldr	r1, =GPDR0_value
	str	r1, [r0, #GPDR0_offset]

	ldr	r2, =GPDR1_value
	str	r2, [r0, #GPDR1_offset]

	ldr	r1, =GPDR2_value
	str	r1, [r0, #GPDR2_offset]

	ldr	r2, =GPDR3_value
	str	r2, [r0, #GPDR3_offset]

	mov	pc, lr

/* Memory setup */
.globl lowlevel_init
lowlevel_init:
	mov	r10, lr

	bl	GPIO_init
	bl	MFPR_init

	/* set CSADRCFG[0] to data flash SRAM mode */
	ldr	r1, =CSADRCFG0
	ldr	r2, =0x0012c919
	str	r2, [r1]	@ WRITE
	ldr	r2, [r1]	@ DELAY UNTIL WRITTEN

	/* Set board voltages */
	bl	init_board_voltage

	/* Configure nCS0 as a VLIO region */
	ldr	r0, =MSC0
	ldr	r1, [r0]	@ Get the MSC0 value
	orr	r1, r1,	#0x9	@ Set the RBW0/RT0 bit
	str	r1, [r0]	@ Update register
	ldr	r1, [r0]	@ Insure write is complete

	mov	lr, r10
	mov	pc, lr
