/*
 * board/cmx300/mfp.h
 *
 * MFP for the CompuLab CM-X300 module
 *
 * (C) Copyright 2009
 * Igor Grinberg, CompuLab, grinberg@compulab.co.il
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CM_X300_MFP__
#define __CM_X300_MFP__


#include <common.h>
#include <asm/arch/pxa-regs.h>
#include <asm/arch/hardware.h>
#include "cmx300_common.h"

/* MFP Masks */
#define MFPR_DEFAULT		(0x0840)
#define MFPR_AF_MASK		(0x0007)
#define MFPR_DRV_MASK		(0x1c00)
#define MFPR_RDH_MASK		(0x0200)
#define MFPR_LPM_MASK		(0xe180)
#define MFPR_PULL_MASK		(0xe000)
#define MFPR_EDGE_MASK		(0x0070)

/* MFP Register Bits */
#define MFP_PULL_SEL		(1 << 15)
#define MFP_PULLUP_EN		(1 << 14)
#define MFP_PULLDOWN_EN		(1 << 13)
#define MFP_DRIVE		(0x7 << 10)
#define MFP_DRIVE_S		10		/* shift */
#define MFP_SLEEP_SEL		(1 << 9)
#define MFP_SLEEP_DATA		(1 << 8)
#define MFP_SLEEP_OE_N		(1 << 7)
#define MFP_EDGE_CLEAR		(1 << 6)
#define MFP_EDGE_FALL_EN	(1 << 5)
#define MFP_EDGE_RISE_EN	(1 << 4)
#define MFP_AF_SEL		(0x7 << 0)

/* MFP Alternate Functions */
#define MFP_AF0			(0)
#define MFP_AF1			(1)
#define MFP_AF2			(2)
#define MFP_AF3			(3)
#define MFP_AF4			(4)
#define MFP_AF5			(5)
#define MFP_AF6			(6)
#define MFP_AF7			(7)

/* MFP Drive */
#define MFP_DS01X		(0)
#define MFP_DS02X		(1)
#define MFP_DS03X		(2)
#define MFP_DS04X		(3)
#define MFP_DS06X		(4)
#define MFP_DS08X		(5)
#define MFP_DS10X		(6)
#define MFP_DS12X		(7)

/* MFP definitions */
#define MFP_BASE	(0x40E10000)

/* MFPR offsets in the MFPR offset tables */

enum MFPRs {
	MFPR_GPIO0 = 0,
	/* CS2 & CS3 are muxed with GPIO1 and GPIO2 accordingly */
	MFPR_GPIO1 = 1,
	MFPR_GPIO2 = 2,

	MFPR_GPIO3 = 3,
	MFPR_GPIO4 = 4,
	MFPR_GPIO5 = 5,
	MFPR_GPIO6 = 6,
	MFPR_GPIO7 = 7,
	MFPR_GPIO8 = 8,
	MFPR_GPIO9 = 9,
	MFPR_GPIO10 = 10,
	MFPR_GPIO11 = 11,
	MFPR_GPIO12 = 12,
	MFPR_GPIO13 = 13,
	MFPR_GPIO14 = 14,
	MFPR_GPIO15 = 15,

	/* CM-X300 revision gpios: 16,18 and 108 below */
	MFPR_GPIO16 = 16,
	MFPR_GPIO17 = 17,
	MFPR_GPIO18 = 18,
	MFPR_GPIO19 = 19,
	MFPR_GPIO20 = 20,
	MFPR_GPIO21 = 21,
	MFPR_GPIO22 = 22,
	MFPR_GPIO23 = 23,
	MFPR_GPIO24 = 24,
	MFPR_GPIO25 = 25,
	MFPR_GPIO26 = 26,
	MFPR_GPIO27 = 27,
	MFPR_GPIO28 = 28,
	MFPR_GPIO29 = 29,
	MFPR_GPIO30 = 30,
	MFPR_GPIO31 = 31,
	MFPR_GPIO32 = 32,
	MFPR_GPIO33 = 33,
	MFPR_GPIO34 = 34,
	MFPR_GPIO35 = 35,
	MFPR_GPIO36 = 36,
	MFPR_GPIO37 = 37,
	MFPR_GPIO38 = 38,
	MFPR_GPIO39 = 39,
	MFPR_GPIO40 = 40,
	MFPR_GPIO41 = 41,
	MFPR_GPIO42 = 42,
	MFPR_GPIO43 = 43,
	MFPR_GPIO44 = 44,
	MFPR_GPIO45 = 45,
	MFPR_GPIO46 = 46,
	MFPR_GPIO47 = 47,
	MFPR_GPIO48 = 48,
	MFPR_GPIO49 = 49,
	MFPR_GPIO50 = 50,
	MFPR_GPIO51 = 51,
	MFPR_GPIO52 = 52,
	MFPR_GPIO53 = 53,

	/* LCD Data */
	MFPR_GPIO54 = 54,	/* LCD Data 0 */
	MFPR_GPIO55 = 55,	/* LCD Data 1 */
	MFPR_GPIO56 = 56,	/* LCD Data 2 */
	MFPR_GPIO57 = 57,	/* LCD Data 3 */
	MFPR_GPIO58 = 58,	/* LCD Data 4 */
	MFPR_GPIO59 = 59,	/* LCD Data 5 */
	MFPR_GPIO60 = 60,	/* LCD Data 6 */
	MFPR_GPIO61 = 61,	/* LCD Data 7 */
	MFPR_GPIO62 = 62,	/* LCD Data 8 */
	MFPR_GPIO63 = 63,	/* LCD Data 9 */
	MFPR_GPIO64 = 64,	/* LCD Data 10 */
	MFPR_GPIO65 = 65,	/* LCD Data 11 */
	MFPR_GPIO66 = 66,	/* LCD Data 12 */
	MFPR_GPIO67 = 67,	/* LCD Data 13 */
	MFPR_GPIO68 = 68,	/* LCD Data 14 */
	MFPR_GPIO69 = 69,	/* LCD Data 15 */
	MFPR_GPIO70 = 70,	/* LCD Data 16 */
	MFPR_GPIO71 = 71,	/* LCD Data 17 */

	/* LCD Clocks */
	MFPR_GPIO72 = 72,	/* LCD FCLK */
	MFPR_GPIO73 = 73,	/* LCD LCLK */
	MFPR_GPIO74 = 74,	/* LCD PCLK */
	MFPR_GPIO75 = 75,	/* LCD BIAS */

	MFPR_GPIO76 = 76,

	/* USB PORT 3 */
	MFPR_GPIO77 = 77,
	MFPR_GPIO78 = 78,
	MFPR_GPIO79 = 79,
	MFPR_GPIO80 = 80,
	MFPR_GPIO81 = 81,
	MFPR_GPIO82 = 82,

	MFPR_GPIO83 = 83,
	MFPR_GPIO84 = 84,
	MFPR_GPIO85 = 85,
	MFPR_GPIO86 = 86,
	MFPR_GPIO87 = 87,
	MFPR_GPIO88 = 88,
	MFPR_GPIO89 = 89,
	MFPR_GPIO90 = 90,
	MFPR_GPIO91 = 91,
	MFPR_GPIO92 = 92,
	MFPR_GPIO93 = 93,
	MFPR_GPIO94 = 94,

	/* V3020 RTC */
	MFPR_GPIO95 = 95,
	MFPR_GPIO96 = 96,
	MFPR_GPIO97 = 97,
	MFPR_GPIO98 = 98,

	MFPR_GPIO99 = 99,
	MFPR_GPIO100 = 100,
	MFPR_GPIO101 = 101,
	MFPR_GPIO102 = 102,
	MFPR_GPIO103 = 103,
	MFPR_GPIO104 = 104,
	MFPR_GPIO105 = 105,
	MFPR_GPIO106 = 106,
	MFPR_GPIO107 = 107,

	/* CM-X300 revision gpios: 108 and 16, 18 above */
	MFPR_GPIO108 = 108,

	MFPR_GPIO109 = 109,
	MFPR_GPIO110 = 110,
	MFPR_GPIO111 = 111,
	MFPR_GPIO112 = 112,
	MFPR_GPIO113 = 113,
	MFPR_GPIO114 = 114,
	MFPR_GPIO115 = 115,
	MFPR_GPIO116 = 116,
	MFPR_GPIO117 = 117,
	MFPR_GPIO118 = 118,
	MFPR_GPIO119 = 119,
	MFPR_GPIO120 = 120,
	MFPR_GPIO121 = 121,
	MFPR_GPIO122 = 122,
	MFPR_GPIO123 = 123,
	MFPR_GPIO124 = 124,
	MFPR_GPIO125 = 125,
	MFPR_GPIO126 = 126,

	/* USB2 PHY reset pin */
	MFPR_GPIO127 = 127,

	MFPR_GPIO0_2 = 128,
	MFPR_GPIO1_2 = 129,
	MFPR_GPIO2_2 = 130,
	MFPR_GPIO3_2 = 131,
	MFPR_GPIO4_2 = 132,
	MFPR_GPIO5_2 = 133,
	MFPR_GPIO6_2 = 134,
	MFPR_GPIO7_2 = 135,
	MFPR_GPIO8_2 = 136,
	MFPR_GPIO9_2 = 137,
	MFPR_GPIO10_2 = 138,
	/* ULPI pins */
	MFPR_ULPI_STP = 139,
	MFPR_ULPI_NXT = 140,
	MFPR_ULPI_DIR = 141,
};

void mfpr_write(enum MFPRs mfpr, u32 value);
u32 mfpr_read(enum MFPRs mfpr);

#endif /* __CM_X300_MFP__ */
