/*
 * (C) Copyright 2002
 * Kyle Harris, Nexus Technologies, Inc. kharris@nexus-tech.net
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * Configuation settings for CompuLab cm-x300 board.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

/*
 * High Level Configuration Options
 * (easy to change)
 */
#define CONFIG_CPU_MONAHANS	1	/* Intel Monahan CPU */
#define CONFIG_CMX300		1	/* CM-X300 board */
#define MONAHANS_L			/* PXA300 processor */
#define MONAHANS_LV			/* PXA310 processor */

#define CONFIG_SKIP_RELOCATE_UBOOT
#define CONFIG_SKIP_LOWLEVEL_INIT
#define CONFIG_BOARD_ENV_READY		1
#define CONFIG_MISC_INIT_R		1
#define BOARD_LATE_INIT			1

#undef	CONFIG_SYS_CLKS_IN_HZ		/* everything, incl board info, in Hz */
#define CONFIG_SYS_HZ			1000

/* Monahans Core Frequency */
#define CONFIG_SYS_MONAHANS_RUN_MODE_OSC_RATIO		16	/* valid values: 8, 16, 24, 31 */
#define CONFIG_SYS_MONAHANS_TURBO_RUN_MODE_RATIO	2	/* valid values: 1, 2 */

#define CONFIG_SETUP_MEMORY_TAGS	1
#define CONFIG_CMDLINE_TAG		1
#define CONFIG_INITRD_TAG		1
#define CONFIG_SERIAL_TAG		1
#define CONFIG_REVISION_TAG		1

#undef CONFIG_USE_IRQ			/* we don't need IRQ/FIQ stuff */

#undef CONFIG_ENV_OVERWRITE

/* serial console configuration */
#define CONFIG_STUART			1
#define CONFIG_SYS_CONSOLE_INFO_QUIET	1
#define CONFIG_SILENT_CONSOLE		1

/* allow to overwrite serial and ethaddr */
/* #define CONFIG_ENV_OVERWRITE */
#define CONFIG_BAUDRATE		38400

/*
 * Size of malloc() pool
 */
#define CONFIG_SYS_MALLOC_LEN		(CONFIG_ENV_SIZE + 128*1024)
#define CONFIG_SYS_GBL_DATA_SIZE	128	/* size in bytes reserved for initial data */

/*
 * Hardware drivers
 */
/* LCD */
#define CONFIG_LCD
#define CONFIG_BOARD_LCD_INIT
#define CONFIG_PXA_BOARD_LCD_MFPR_GPIO_SETUP
#define CONFIG_TD035STEE1
#define BOARD_SPLASH_SCREEN_PREPARE
#define CONFIG_SPLASH_SCREEN

/* I2C */
#define CONFIG_HARD_I2C
#define CONFIG_SYS_I2C_SPEED	400000
#define CONFIG_SYS_I2C_SLAVE	0x1
#define CONFIG_PCA953X
#define CONFIG_PCA953X_16BIT
#define CONFIG_PCA953X_INFO

/* MMC */
#define CONFIG_MMC			1
#define CONFIG_CMD_MMC			1
#define CONFIG_PXA_MMC			1
#define CONFIG_PXA_MMC_BOARD_INIT	1
#define CONFIG_SYS_MMC_BASE		0xf0000000

/* NET */
#define CONFIG_NET_MULTI
#define CONFIG_DRIVER_DM9000
#define CONFIG_HAS_ETH1
#define CONFIG_DM9000_BASE	0x08000010
#define CONFIG_EXT_DM9000_BASE	0x14000010
#define DM9000_IO		(CONFIG_DM9000_BASE)
#define DM9000_DATA		(CONFIG_DM9000_BASE + 0x4)
#define EXT_DM9000_IO		(CONFIG_EXT_DM9000_BASE)
#define EXT_DM9000_DATA		(CONFIG_EXT_DM9000_BASE + 0x4)
#define DM9000_NO_EEPROM
#define CONFIG_DM9000_USE_16BIT
#define DM9000_KEEP_PHY_ON

/*
 * Command line configuration.
 */
#include <config_cmd_default.h>

#define CONFIG_CMD_ASKENV
#define CONFIG_CMD_DHCP
#define CONFIG_CMD_EXT2
#define CONFIG_CMD_FAT
#define CONFIG_CMD_NAND
#define CONFIG_CMD_JFFS2
#define CONFIG_CMD_PING
#define CONFIG_CMD_USB
#define CONFIG_CMD_BMP

#define CONFIG_CMD_I2C
#define CONFIG_I2C_CMD_TREE
#define CONFIG_CMD_PCA953X
#define CONFIG_CMD_PCA953X_INFO

#undef CONFIG_CMD_FLASH
#undef CONFIG_CMD_IMLS

#define CONFIG_BOOTDELAY	3
#define CONFIG_SYS_AUTOLOAD	"no"
#define CONFIG_NETMASK		255.255.255.0
#define CONFIG_IPADDR		10.1.1.97
#define CONFIG_SERVERIP		10.1.1.95
#define CONFIG_BOOTCOMMAND	"fsload ; bootm"
#define CONFIG_BOOTARGS		"root=/dev/ram0 console=ttyS2,38400"

/* #define CONFIG_EXTRA_ENV_SETTINGS \ */
/*         "update_uboot=tftp 80a00000 x300-bl-image; nand erase 0 80000; nand write 80a00000 0 80000\0" */

#define CONFIG_TIMESTAMP

#if defined(CONFIG_CMD_KGDB)
#define CONFIG_KGDB_BAUDRATE	115200		/* speed to run kgdb serial port */
#define CONFIG_KGDB_SER_INDEX	2		/* which serial port to use */
#endif

/*
 * Miscellaneous configurable options
 */
#define CONFIG_SYS_HUSH_PARSER		1
#define CONFIG_AUTO_COMPLETE		1
#define CONFIG_CMDLINE_EDITING		1
#define CONFIG_SYS_LONGHELP		1		/* undef to save memory	*/
#define CONFIG_SYS_PROMPT		"U-Boot> "	/* Monitor Command Prompt */
#define CONFIG_SYS_PROMPT_HUSH_PS2	"> "
#define CONFIG_SYS_CBSIZE		1024		/* Console I/O Buffer Size */
/* Print Buffer Size */
#define CONFIG_SYS_PBSIZE 		(CONFIG_SYS_CBSIZE+sizeof(CONFIG_SYS_PROMPT)+16)
#define CONFIG_SYS_MAXARGS		16		/* max number of command args */
#define CONFIG_SYS_BARGSIZE		CONFIG_SYS_CBSIZE	/* Boot Argument Buffer Size */
#define CONFIG_SYS_DEVICE_NULLDEV	1

#define CONFIG_SYS_MEMTEST_START	0x80400000	/* memtest works on	*/
#define CONFIG_SYS_MEMTEST_END		0x80800000	/* 4 ... 8 MB in DRAM	*/
/* default load address */
#define CONFIG_SYS_LOAD_ADDR		(CONFIG_SYS_DRAM_BASE + 0x808000)


/* valid baudrates */
#define CONFIG_SYS_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

/*
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	(128*1024)	/* regular stack */
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ	(4*1024)	/* IRQ stack */
#define CONFIG_STACKSIZE_FIQ	(4*1024)	/* FIQ stack */
#endif

/*
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS	2		/* we have 2 banks of DRAM */
#define PHYS_SDRAM_1		0x80000000	/* SDRAM Bank #1 */
#define PHYS_SDRAM_1_SIZE	0x04000000	/* 64 MB */
#define PHYS_SDRAM_2		0xc0000000	/* SDRAM Bank #2 */
#define PHYS_SDRAM_2_SIZE	0x04000000	/* 64 MB */

#define CONFIG_SYS_DRAM_BASE	0x80000000	/* at CS0 */
#define CONFIG_SYS_DRAM_SIZE	0x08000000	/* 128 MB Ram */

/*
 * NAND Flash
 */

#define CONFIG_NEW_NAND_CODE
#define CONFIG_SYS_NAND0_BASE		0x0
#undef CONFIG_SYS_NAND1_BASE

#define CONFIG_SYS_NAND_BASE_LIST	{ CONFIG_SYS_NAND0_BASE }
#define CONFIG_SYS_MAX_NAND_DEVICE	1	/* Max number of NAND devices */

/* nand timeout values */
#define CONFIG_SYS_NAND_PROG_ERASE_TO	3000
#define CONFIG_SYS_NAND_OTHER_TO	400
#define CONFIG_SYS_NAND_SENDCMD_RETRY	3
#undef NAND_ALLOW_ERASE_ALL	/* Allow erasing bad blocks - don't use */

/* NAND Timing Parameters (in ns) */
#define NAND_TIMING_tCH		5
#define NAND_TIMING_tCS		20
#define NAND_TIMING_tWH		10
#define NAND_TIMING_tWP		12

#define NAND_TIMING_tRH		10
#define NAND_TIMING_tRP		12

#define NAND_TIMING_tR		20000
#define NAND_TIMING_tWHR	60
#define NAND_TIMING_tAR		10

/* This has no use with Linux pxa3xx nand driver */
/* NAND debugging */
#undef CONFIG_SYS_DFC_DEBUG1	/* usefull */
#undef CONFIG_SYS_DFC_DEBUG2	/* noisy */
#undef CONFIG_SYS_DFC_DEBUG3	/* extremly noisy  */

/* #define CONFIG_MTD_DEBUG */
/* #define CONFIG_MTD_DEBUG_VERBOSE 1 */

#define ADDR_COLUMN		1
#define ADDR_PAGE		2
#define ADDR_COLUMN_PAGE	3

#define NAND_ChipID_UNKNOWN		0x00
#define NAND_MAX_FLOORS			1
#define CONFIG_SYS_NAND_MAX_CHIPS	1

#define CONFIG_SYS_NO_FLASH		1

#define CONFIG_ENV_IS_IN_NAND		1
#define CONFIG_ENV_OFFSET		0x80000

/* #define CONFIG_ENV_OFFSET_REDUND	0xa0000 */
#define CONFIG_ENV_SIZE         	0x20000		/* 1 sector = 128 kB */

/* JFFS2 */
#define CONFIG_JFFS2_NAND		1
#define CONFIG_JFFS2_DEV		"nand0"
#define CONFIG_JFFS2_NAND_DEV		0		/* nand device jffs2 lives on */
#define CONFIG_JFFS2_PART_OFFSET	2*1024*1024	/* start of jffs2 partition */
#define CONFIG_JFFS2_PART_SIZE		4*1024*1024	/* size of jffs2 partition */

/*
 * USB OHCI settings
 */
#define OHCI_REGS_BASE				0x4C000000
#define CONFIG_SYS_DEVICE_DEREGISTER		1
#define CONFIG_USB_OHCI_NEW			1
#define CONFIG_USB_STORAGE			1
#define CONFIG_DOS_PARTITION			1
#define CONFIG_SYS_USB_OHCI_BOARD_INIT		1
#define CONFIG_SYS_USB_OHCI_CPU_INIT		1
#define CONFIG_SYS_USB_OHCI_SLOT_NAME		"cmx300"
#define CONFIG_SYS_USB_OHCI_REGS_BASE		OHCI_REGS_BASE
#define CONFIG_SYS_USB_OHCI_MAX_ROOT_PORTS	3
#define LITTLEENDIAN				1	/* used by usb_ohci.c  */

#define CONFIG_PROGRESS_BAR

#endif	/* __CONFIG_H */
